#!/usr/bin/env python
import os, sys, pygame, tarfile, getopt
from pygame.locals import *

sys.path.insert(0, "..")
#sys.path.append("../game/")
#sys.path.append("../game/media/")

import engine, actor, entities
from pgu import gui
from vector import *

class NewDialog(gui.Dialog):
    def __init__(self, **params):
        title = gui.Label("New MapFile")
        self.value = gui.Form()
        layout = gui.Table()
        layout.tr()
        layout.td(gui.Label("Map Image:"))
        self.image_name = gui.Input(name="image_name")
        layout.td(self.image_name)
        i_button = gui.Button("Browse")
        i_button.connect(gui.CLICK, self.open_browser, None)
        layout.td(i_button)
        layout.tr()
        s_button = gui.Button("Okay")
        s_button.connect(gui.CLICK, self.send, gui.CHANGE)
        layout.td(s_button)
        
        gui.Dialog.__init__(self,title,layout)
        
    def open_browser(self, value):
        browser = gui.FileDialog()
        browser.connect(gui.CHANGE, self.close_browser, browser)
        browser.open()
    
    def close_browser(self, rsc):
        if rsc.value: self.image_name.value = rsc.value

class SaveDialog(gui.Dialog):
    def __init__(self, file_name, **params):        
        title = gui.Label("Save MapFile As")
        self.value = gui.Form()
        layout = gui.Table()
        layout.tr()
        layout.td(gui.Label("File Name:"))
        file_name = os.path.abspath(file_name)
        self.file_name = gui.Input(file_name, name="file_name")
        layout.td(self.file_name)
        b_button = gui.Button("Browse")
        b_button.connect(gui.CLICK, self.open_browser, None)
        layout.td(b_button)
        s_button = gui.Button("Okay")
        s_button.connect(gui.CLICK, self.send, gui.CHANGE)
        layout.td(s_button)
        gui.Dialog.__init__(self,title,layout)
    
    def open_browser(self, value):
        browser = gui.FileDialog()
        browser.connect(gui.CHANGE, self.close_browser, browser)
        browser.open()
    
    def close_browser(self, rsc):
        if rsc.value: self.file_name.value = rsc.value
        
class OpenDialog(gui.Dialog):
    def __init__(self, **params):
        title = gui.Label("Open Mapfile")
        self.value = gui.Form()
        layout = gui.Table()
        layout.tr()
        layout.td(gui.Label("Map File Name (.txt):"))
        self.file_name = gui.Input(name="file_name")
        layout.td(self.file_name)
        b_button = gui.Button("Browse")
        b_button.connect(gui.CLICK, self.open_browser, None)
        layout.td(b_button)
        s_button = gui.Button("Okay")
        s_button.connect(gui.CLICK, self.send, gui.CHANGE)
        layout.td(s_button)
        gui.Dialog.__init__(self,title,layout)
    
    def open_browser(self, value):
        browser = gui.FileDialog()
        browser.connect(gui.CHANGE, self.close_browser, browser)
        browser.open()
    
    def close_browser(self, rsc):
        if rsc.value: self.file_name.value = rsc.value
        
class ExitDialog(gui.Dialog):
    def __init__(self, **params):
        title = gui.Label("Exit")
        self.value = gui.Form()
        layout = gui.Table()
        layout.tr()
        layout.add(gui.Label("Are you sure you want to Exit?",colspan=2))
        layout.tr()
        layout.add(gui.Label("(All unsaved changes will be lost!)",colspan=2))
        layout.tr()
        e_button = gui.Button("Exit")
        e_button.connect(gui.CLICK,self.send,gui.QUIT)
        layout.td(e_button)
        
        c_button = gui.Button("Cancel")
        c_button.connect(gui.CLICK,self.close,None)
        layout.td(c_button)
        
        gui.Dialog.__init__(self,title,layout)


class ResourceDialog(gui.Dialog):
    def __init__(self, **params):
        title = gui.Label("Resource")

        self.value = gui.Form()
        layout = gui.Table()
        layout.tr()
        layout.td(gui.Label("Resource Name"),align=0,colspan=3)
        layout.tr()

        rsc_name = gui.Select(name="r_name")
        for item in entities.classVector:
            rsc_name.add(item,item)
        layout.td(rsc_name,colspan=3)
        layout.tr()
        layout.td(gui.Label("KW Args"), colspan=3)
        layout.tr()
        
        kinput= gui.Input(name='kwargs')
        layout.td(kinput,colspan=3)

        layout.tr()
        ok_button = gui.Button("Ok")
        ok_button.connect(gui.CLICK,self.send,gui.CHANGE)
        layout.td(ok_button, col=3)
        gui.Dialog.__init__(self,title,layout)
  
class MapEditor(gui.Widget):
    def __init__(self, image_name, **params):
        gui.Widget.__init__(self, **params)
        self.position = None
        self.surface = None
        self.state = 0
        self.bg_color = pygame.Color('#FFFFFF')
        self.image_name = image_name
        self.engine = None

    def init(self, **params):
        self.width = 200
        self.height = 200
        self.__dict__.update(params)
        self.surface = pygame.Surface((self.width, self.height), pygame.SRCALPHA, 32)
        
    def paint(self,surface):
        if not self.engine:
            self.engine = engine.Engine(self.image_name, None, False, self.surface)
            self.engine.doFrame=self.engine.renderFrame
        self.engine.runFrame(list())
        surface.blit(self.surface,(0,0))
        
    def event(self, ev):
        if not self.surface: return
        if ev.type == gui.MOUSEBUTTONDOWN:
        
            if hasattr(self,app.mode+"_down"):
                action = getattr(self,app.mode+"_down")
                action(ev)
        
        if ev.type == gui.MOUSEMOTION:
        
            if hasattr(self,app.mode+"_motion"):
                action = getattr(self,app.mode+"_motion")
                action(ev)
        
        if ev.type is gui.MOUSEBUTTONUP:
        
            if hasattr(self,app.mode+"_up"):
                action = getattr(self,app.mode+"_up")
                action(ev)
        
        if ev.type is gui.KEYDOWN:
            if ev.key == gui.K_w:
                self.engine.scrollPosition=subv(self.engine.scrollPosition, (0,64))
                self.repaint()
            if ev.key == gui.K_s:
                self.engine.scrollPosition=addv(self.engine.scrollPosition, (0,64))
                self.repaint()
            if ev.key == gui.K_a:
                self.engine.scrollPosition=subv(self.engine.scrollPosition, (64,0))
                self.repaint()
            if ev.key == gui.K_d:
                self.engine.scrollPosition=addv(self.engine.scrollPosition, (64,0))
                self.repaint()
    """
        if ev.type is gui.KEYUP:
            if ev.key == gui.K_UP:
                print "Up Arrow Up!"
            if ev.key == gui.K_DOWN:
                print "Down Arrow Up!"
            if ev.key == gui.K_LEFT:
                print "Left Arrow Up!"
            if ev.key == gui.K_RIGHT:
                print "stop with the right"
    """            
                
    def add_down(self, ev):
        self.state=1

    def add_move(self, ev):
        if self.state == 0: return
        
    def add_up(self, ev):
        self.state=0
        if not app.mapinfo: return
        print ev.pos
        pos = self.engine.screenToMap(ev.pos)
        if app.mapinfo[-1].has_key('pos'):
            r_name = app.mapinfo[-1]['r_name']
            if app.mapinfo[-1]['kwargs']:
                kwargs = app.mapinfo[-1]['kwargs']
            else:
                kwargs = ''
            
            self.insert_entity(r_name,pos,kwargs)
            app.mapinfo.append({'kwargs':kwargs,'pos':pos,'r_name':r_name})
        else:
            app.mapinfo[-1]['pos'] = pos
            if app.mapinfo[-1]['kwargs']:
                kwargs = app.mapinfo[-1]['kwargs']
            else:
                kwargs = ''
            self.insert_entity(app.mapinfo[-1]['r_name'],app.mapinfo[-1]['pos'],kwargs)
        #print "Add Up!", app.rsc_mapinfo[-1]
    
    def load_from_file(self):
        if not app.mapinfo: return
        for ent in app.mapinfo:
            self.insert_entity(ent['r_name'],ent['pos'],ent['kwargs'])
    
    def insert_entity(self, r_name, pos, kwargs):
        self.engine.insertEntity(r_name, pos, kwargs)
        self.repaint()
        
    def do_add(self):
        self.dialog.close()
        
    def edit_down(self, ev):
        self.state=1
        print "Edit Down!"
    def edit_move(self, ev):
        if self.state == 0: return
        print "Edit Move!"
    def edit_up(self, ev):
        self.state=0
        print "Edit Up!"
        
    def move_down(self, ev):
        self.state=1
        print "Move Down!"
    def move_move(self, ev):
        if self.state == 0: return
        print "Move Move!"
    def move_up(self, ev):
        self.state=0
        print "Move Up!"
        
class App(gui.Desktop):
    def __init__(self, fname, **params):
        gui.Desktop.__init__(self,**params)
        self.connect(gui.QUIT,self.quit,None)
        self.mapinfo = list()
        
        gc = gui.Container(width=800, height=600)
        
        self.image_name = fname
        self.file_name = os.path.splitext(self.image_name)[0]+'.txt'
        self.new_dialog = NewDialog()
        self.new_dialog.connect(gui.CHANGE,self.do_new,None)
        self.open_dialog = OpenDialog()
        self.open_dialog.connect(gui.CHANGE,self.do_open,None)
        self.save_dialog = SaveDialog(self.file_name)
        self.save_dialog.connect(gui.CHANGE, self.do_saveas,None)
        self.exit_dialog = ExitDialog()
        self.exit_dialog.connect(QUIT,self.quit,None)
        
        self.menus = gui.Menus([
            ('File/Open',self.open_dialog.open,None),
            ('File/Save As',self.save_dialog.open,None),
            ('File/Exit',self.exit_dialog.open,None)
            ])
        """('File/New',self.new_dialog.open,None),('File/Save',self.do_save,None),"""
        gc.add(self.menus,0,0)
        self.menus.rect.w,self.menus.rect.h = self.menus.resize()
        
        self.mode = 'add'
        
        mode = gui.Table()
        add = gui.Button(gui.Label("Add"), width=100, height=20)
        add.connect(gui.CLICK,self.engage_mode,'add')
        mode.tr()
        mode.td(add)
        edit = gui.Button(gui.Label("Edit"), width=100, height=20)
        edit.connect(gui.CLICK,self.engage_mode,'edit')
        mode.tr()
        mode.td(edit)
        move = gui.Button(gui.Label("Move"), width=100, height=20)
        move.connect(gui.CLICK,self.engage_mode,'move')
        mode.tr()
        mode.td(move)
        gc.add(mode,0,self.menus.rect.bottom+5)
        mode.rect.x,mode.rect.y = mode.style.x,mode.style.y
        mode.rect.w,mode.rect.h = mode.resize()
        
        map_editor = MapEditor(self.image_name,width=640,height=480, style={'border':1})
        map_editor.init(width=640,height=480)
        self.map_editor = map_editor
        gc.add(map_editor,mode.rect.w+10,self.menus.rect.h+5)
        
        self.widget = gc
        
    def do_open(self, value):
        self.open_dialog.close()
        self.file_name = self.open_dialog.value['file_name'].value
        print "Map File Name: " + self.file_name
        fh = open(self.file_name, "r")
        lines = fh.readlines()
        for line in lines:
            bundle = line.split(":")
            print "bundle: ",bundle
            if len(bundle) == 1:
                map_image_name = bundle[0]
                print "map image name: ",map_image_name
            else:
                rname, pos, kwargs = bundle
                exec('posn='+pos)
                self.mapinfo.append({'r_name':rname,'pos':posn,'kwargs':kwargs})
        fh.close()
        self.map_editor.load_from_file()

    def do_save(self, value):
        fh = open(self.file_name, "w")
        for i in self.mapinfo:
            #print i
            str = i['r_name'] + ":" + i['pos'].__repr__() + ":" + i['kwargs'] + "\n"
            #print str
            fh.write(str)
        #print "Save Name: " + self.file_name
        fh.close()
    
    def do_saveas(self, value):
        self.save_dialog.close()
        self.file_name = self.save_dialog.value['file_name'].value
        
        #print "Save As Name: " + self.file_name
        self.do_save(self.file_name)
    
    def do_new(self, value):
        self.new_dialog.close()
        self.image_name = self.new_dialog.value['image_name'].value
        print "New Image Name: " + self.image_name
        self.file_name = os.path.splitext(self.image_name)[0]+'.txt'
        print "Mape Name: " + self.file_name
    
    def engage_mode(self, value):
        self.mode = value
        
        if self.mode == 'add':
            rsd = ResourceDialog()
            rsd.connect(gui.CHANGE, self.add_info, rsd)
            rsd.open()
        if self.mode == 'edit':
            print 'edit'
        if self.mode == 'move':
            print 'move'
    
    def add_info(self, rsc):
        rsc.close()
        #print "r_name",rsc.value['r_name'].value
        #print "kwargs",rsc.value['kwargs'].value
        ri = {'r_name':rsc.value['r_name'].value,'kwargs':rsc.value['kwargs'].value}
        self.mapinfo.append(ri)
        
def usage():
    print "Edit Map: Start by passing '-f map_image_name.png'"
    print "Example: python map-editor.py -f media/testlevel.png"
    
def main(argv=None):
    if argv is None:
        argv = sys.argv[1:]
    try:
        opts, args = getopt.getopt(argv, "hf:", ["help","file="])
    except getopt.GetoptError:
        usage()
        sys.exit(2)
        
    print opts
    path = os.getcwd()
    
    fname = None
    for opt, arg in opts:
        if opt in ('-h','--help'):
            usage()
        if opt in ('-f','--file'):
            fname = arg
    global app
    app = App(fname)
    app.run()
    
if __name__ == "__main__":
    main(sys.argv[1:])